package com.incubator.core.net.ws;

import com.incubator.common.net.Connection;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 请求过滤器
 */
public class RequestFilter {

    // 存储最近的请求时间戳：<请求唯一标识, 上次请求时间>
    private static final Map<String, Long> requestTimestampMap = new ConcurrentHashMap<>();

    // 最短间隔时间（单位：毫秒）
    private static final long MIN_REQUEST_INTERVAL = 1000; // 1秒

    /**
     * 检查是否允许处理请求
     *
     * @param conn 当前玩家连接
     * @param cmd     消息命令号
     * @return true 允许处理，false 拒绝处理
     */
    public static boolean isAllowed(Connection conn, int cmd) {
        String key = generateRequestKey(conn, cmd);
        long now = System.currentTimeMillis();

        // 检查上次请求时间
        Long lastRequestTime = requestTimestampMap.get(key);

        if (lastRequestTime != null && (now - lastRequestTime < MIN_REQUEST_INTERVAL)) {
            // 请求过于频繁，拒绝处理
            return false;
        }

        // 更新请求时间
        requestTimestampMap.put(key, now);
        return true;
    }

    /**
     * 生成请求的唯一标识
     */
    private static String generateRequestKey(Connection conn, int cmd) {
        return conn + ":" + cmd;
    }
}
